import icedata
from icevision.all import *


def test_parser(data_dir):
    class_map = icedata.pets.class_map()
    parser = icedata.pets.parser(data_dir, class_map=class_map, mask=True)

    records = parser.parse()[0]
    assert len(records) == 5
    record = records[0]

    expected = {"imageid", "labels", "bboxes", "filepath", "height", "width", "masks"}
    assert set(record.keys()) == expected

    assert record["filepath"].name == "Abyssinian_119.jpg"
    assert record["imageid"] == 0
    assert record["labels"] == [1]
    assert record["height"] == 297
    assert record["width"] == 300

    assert record["bboxes"] == [BBox.from_xyxy(39, 51, 156, 179)]
    assert record["masks"][0].filepath.name == "Abyssinian_119.png"



import icedata


def test_class_map():
    num_classes = icedata.coco.NUM_CLASSES
    class_map = icedata.coco.class_map()

    assert num_classes == len(class_map) == 91
    assert class_map.get_id(0) == "background"
    assert class_map.get_id(-1) == "toothbrush"



__all__ = ["parser"]

from icevision.imports import *
from icevision.core import *
from icevision import parsers


def parser(data_dir: Path, class_map: ClassMap):
    parser = parsers.VocXmlParser(
        annotations_dir=data_dir / "odFridgeObjects/annotations",
        images_dir=data_dir / "odFridgeObjects/images",
        class_map=class_map,
    )

    return parser



__all__ = ["load_model_weights_from_url"]

from icevision.all import *


def load_model_weights_from_url(
    model: nn.Module, url: str, map_location=torch.device("cpu"), **kwargs
) -> None:
    state_dict = torch.hub.load_state_dict_from_url(
        url, map_location=map_location, **kwargs
    )
    model.load_state_dict(state_dict)



__all__ = ["parser", "PetsXmlParser", "PetsMaskParser", "PetsMaskFile"]

from icevision.imports import *
from icevision.core import *
from icevision import parsers


def parser(data_dir: Path, class_map: ClassMap, mask=False):
    parser = PetsXmlParser(
        annotations_dir=data_dir / "annotations/xmls",
        images_dir=data_dir / "images",
        class_map=class_map,
    )

    if mask:
        mask_parser = PetsMaskParser(data_dir / "annotations/trimaps")
        parser = parsers.CombinedParser(parser, mask_parser)

    return parser


class PetsXmlParser(parsers.VocXmlParser):
    def labels(self, o) -> List[int]:
        name = re.findall(r"^(.*)_\d+$", o.stem)[0]
        class_id = self.class_map.get_name(name)

        # there is an image with two cats (same breed)
        num_objs = len(self._root.findall("object"))

        return [class_id] * num_objs


class PetsMaskParser(parsers.VocMaskParser):
    def masks(self, o) -> List[Mask]:
        return [PetsMaskFile(o)]


@dataclass
class PetsMaskFile(VocMaskFile):
    """Extension of `MaskFile` for Pets masks.
    Invert 0s and 1s in the mask (the background is orignally 1 in the pets masks)
    Removes the color pallete and optionally drops void pixels.

    Args:
          drop_void (bool): drops the void pixels, which should have the value 255.
    """

    def to_mask(self, h, w) -> MaskArray:
        mask = super().to_mask(h=h, w=w)
        mask.data = 1 - mask.data
        return mask



import icedata
from icevision.all import *


def test_parser(data_dir):
    class_map = icedata.fridge.class_map()
    parser = icedata.fridge.parser(data_dir, class_map=class_map)

    records = parser.parse()[0]
    assert len(records) == 5
    record = records[0]

    expected = {"imageid", "labels", "bboxes", "filepath", "height", "width"}
    assert set(record.keys()) == expected

    assert record["filepath"].name == "10.jpg"
    assert record["imageid"] == 0
    assert record["labels"] == [2, 3]
    assert record["height"] == 666
    assert record["width"] == 499

    expected = [BBox.from_xyxy(86, 102, 216, 439), BBox.from_xyxy(150, 377, 445, 490)]
    assert record["bboxes"] == expected



__all__ = ["parser", "ImageParser", "AnnotationParser", "BirdMaskFile"]

from icevision.imports import *
from icevision.utils import *
from icevision.core import *
from icevision import parsers


def parser(data_dir: Union[str, Path], class_map: ClassMap) -> parsers.ParserInterface:
    image_parser = ImageParser(data_dir)
    annotation_parser = AnnotationParser(data_dir, class_map)
    return parsers.CombinedParser(image_parser, annotation_parser)


class ImageParser(parsers.Parser, parsers.FilepathMixin):
    def __init__(self, data_dir):
        self.image_filepaths = get_image_files(data_dir)

    def __iter__(self) -> Any:
        yield from self.image_filepaths

    def filepath(self, o) -> Union[str, Path]:
        return o

    def imageid(self, o) -> Hashable:
        return o.stem


class AnnotationParser(
    parsers.Parser, parsers.MasksMixin, parsers.BBoxesMixin, parsers.LabelsMixin
):
    def __init__(self, data_dir, class_map):
        self.mat_filepaths = get_files(
            data_dir / "annotations-mat", extensions=[".mat"]
        )
        self.class_map = class_map

    def __iter__(self) -> Any:
        yield from self.mat_filepaths

    def masks(self, o) -> List[Mask]:
        return [BirdMaskFile(o)]

    def bboxes(self, o) -> List[BBox]:
        import scipy.io

        mat = scipy.io.loadmat(str(o))
        bbox = mat["bbox"]
        xyxy = [int(bbox[pos]) for pos in ["left", "top", "right", "bottom"]]
        return [BBox.from_xyxy(*xyxy)]

    def imageid(self, o) -> Hashable:
        return o.stem

    def labels(self, o) -> List[int]:
        class_name = o.parent.name
        return [self.class_map.get_name(class_name)]


class BirdMaskFile(MaskFile):
    def to_mask(self, h, w):
        import scipy.io

        mat = scipy.io.loadmat(str(self.filepath))
        return MaskArray(mat["seg"])[None]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['icedata',
 'icedata.datasets',
 'icedata.datasets.birds',
 'icedata.datasets.coco',
 'icedata.datasets.fridge',
 'icedata.datasets.fridge.tests',
 'icedata.datasets.pennfudan',
 'icedata.datasets.pennfudan.tests',
 'icedata.datasets.pets',
 'icedata.datasets.pets.tests',
 'icedata.datasets.voc',
 'icedata.utils']

package_data = \
{'': ['*'],
 'icedata.datasets.coco': ['tests/*'],
 'icedata.datasets.fridge': ['sample_data/odFridgeObjects/annotations/*',
                             'sample_data/odFridgeObjects/images/*'],
 'icedata.datasets.pennfudan': ['sample_data/Annotation/*',
                                'sample_data/PNGImages/*',
                                'sample_data/PedMasks/*'],
 'icedata.datasets.pets': ['images/*',
                           'sample_data/annotations/images/*',
                           'sample_data/annotations/trimaps/*',
                           'sample_data/annotations/xmls/*']}

install_requires = \
['icevision>=0.1.0-rc.1,<0.2.0']

setup_kwargs = {
    'name': 'icedata',
    'version': '0.0.2rc3',
    'description': 'Datasets collection',
    'long_description': None,
    'author': 'Lucas Goulart Vazquez',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



__all__ = ["parser", "PennFundanParser"]

from icevision.imports import *
from icevision import *


def parser(data_dir) -> parsers.ParserInterface:
    return PennFundanParser(data_dir=data_dir)


class PennFundanParser(parsers.MaskRCNN, parsers.FilepathMixin, parsers.SizeMixin):
    def __init__(self, data_dir):
        self.data_dir = data_dir
        self.filenames = get_files(data_dir / "Annotation", extensions=".txt")

    def __iter__(self):
        yield from self.filenames

    def __len__(self):
        return len(self.filenames)

    def prepare(self, o):
        self._imageid = getattr(self, "_imageid", 0) + 1
        self.lines = L(o.read().split("\n"))
        self._bboxes = []

        for line in self.lines:
            if line.startswith("Image filename"):
                filename = re.findall(r'"(.*)"', line)[0]
                self._filepath = self.data_dir.parent / filename

            elif line.startswith("Image size (X x Y x C)"):
                size_str = re.search(r"\d{3,4}\sx\s\d{3,4}\sx\s3", line).group()
                self._size = [int(o) for o in size_str.split("x")]

            elif line.startswith("Objects with ground truth"):
                # number of objects is the first number that shows in the line
                self._num_objects = int(re.findall("\d+", line)[0])

            elif line.startswith("Pixel mask for object"):
                mask_filename = re.findall(r'"(.+)"', line.split(":")[-1])[0]
                self._mask_filepath = self.data_dir.parent / mask_filename

            if line.startswith("Bounding box"):
                # find bbox coordinates in line and covert to a list
                point_pairs_str = re.findall(r"(\d+,\s\d+)", line)
                points = []
                for pairs in point_pairs_str:
                    for point in pairs.split(","):
                        points.append(int(point))

                bbox = BBox.from_xyxy(*points)
                self._bboxes.append(bbox)

    def imageid(self, o) -> int:
        return self._imageid

    def filepath(self, o) -> Union[str, Path]:
        return self._filepath

    def width(self, o) -> int:
        return self._size[0]

    def height(self, o) -> int:
        return self._size[1]

    def labels(self, o) -> List[int]:
        return [1] * self._num_objects

    def iscrowds(self, o) -> List[bool]:
        return [False] * self._num_objects

    def masks(self, o) -> List[Mask]:
        return [MaskFile(self._mask_filepath)]

    def bboxes(self, o) -> List[BBox]:
        return self._bboxes



import icedata
from icevision.all import *


def test_parser(data_dir):
    class_map = icedata.pennfudan.class_map()
    parser = icedata.pennfudan.parser(data_dir)

    records = parser.parse()[0]
    assert len(records) == 5
    record = records[0]

    expected = {
        "imageid",
        "labels",
        "bboxes",
        "masks",
        "iscrowds",
        "filepath",
        "height",
        "width",
    }
    assert set(record.keys()) == expected

    assert record["filepath"].name == "FudanPed00001.png"
    assert record["imageid"] == 0
    assert record["labels"] == [1, 1]
    assert record["height"] == 536
    assert record["width"] == 559

    expected = [BBox.from_xyxy(160, 182, 302, 431), BBox.from_xyxy(420, 171, 535, 486)]
    assert record["bboxes"] == expected
